# pubsubplus-python-client
#
# Copyright 2021-2023 Solace Corporation. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# 	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# this module contains the CCSMP session properties mapping.
# pylint: disable= missing-module-docstring,line-too-long,protected-access

from solace.messaging.config._sol_constants import SOLCLIENT_PROP_ENABLE_VAL, \
    SOLCLIENT_FLOW_PROP_BIND_ENTITY_DURABLE, SOLCLIENT_FLOW_PROP_ACKMODE, \
    SOLCLIENT_FLOW_PROP_ACKMODE_CLIENT, SOLCLIENT_FLOW_PROP_BIND_ENTITY_ID, \
    SOLCLIENT_FLOW_PROP_BIND_ENTITY_QUEUE, SOLCLIENT_FLOW_PROP_ACTIVE_FLOW_IND, \
    SOLCLIENT_ENDPOINT_PROP_ID, SOLCLIENT_ENDPOINT_PROP_QUEUE, \
    SOLCLIENT_ENDPOINT_PROP_DURABLE, SOLCLIENT_ENDPOINT_PROP_ACCESSTYPE, \
    SOLCLIENT_ENDPOINT_PROP_ACCESSTYPE_EXCLUSIVE, \
    SOLCLIENT_FLOW_PROP_REPLAY_START_LOCATION_BEGINNING, SOLCLIENT_FLOW_PROP_REPLAY_START_LOCATION, \
    SOLCLIENT_SESSION_PROP_HOST, SOLCLIENT_SESSION_PROP_CONNECT_TIMEOUT_MS, SOLCLIENT_SESSION_PROP_CONNECT_RETRIES, \
    SOLCLIENT_SESSION_PROP_CONNECT_RETRIES_PER_HOST, SOLCLIENT_SESSION_PROP_RECONNECT_RETRIES, \
    SOLCLIENT_SESSION_PROP_RECONNECT_RETRY_WAIT_MS, SOLCLIENT_SESSION_PROP_KEEP_ALIVE_INT_MS, \
    SOLCLIENT_SESSION_PROP_KEEP_ALIVE_LIMIT, SOLCLIENT_SESSION_PROP_SOCKET_RCV_BUF_SIZE, \
    SOLCLIENT_SESSION_PROP_SOCKET_SEND_BUF_SIZE, SOLCLIENT_SESSION_PROP_TCP_NODELAY, \
    SOLCLIENT_SESSION_PROP_COMPRESSION_LEVEL, SOLCLIENT_SESSION_PROP_SSL_VALIDATE_CERTIFICATE, \
    SOLCLIENT_SESSION_PROP_SSL_VALIDATE_CERTIFICATE_DATE, SOLCLIENT_SESSION_PROP_SSL_VALIDATE_CERTIFICATE_HOST, \
    SOLCLIENT_SESSION_PROP_SSL_EXCLUDED_PROTOCOLS, SOLCLIENT_SESSION_PROP_SSL_CONNECTION_DOWNGRADE_TO, \
    SOLCLIENT_SESSION_PROP_SSL_CIPHER_SUITES, SOLCLIENT_SESSION_PROP_SSL_TRUST_STORE_DIR, \
    SOLCLIENT_SESSION_PROP_SSL_TRUSTED_COMMON_NAME_LIST, SOLCLIENT_SESSION_PROP_VPN_NAME, \
    SOLCLIENT_SESSION_PROP_GENERATE_SENDER_ID, SOLCLIENT_SESSION_PROP_GENERATE_RCV_TIMESTAMPS, \
    SOLCLIENT_SESSION_PROP_GENERATE_SEND_TIMESTAMPS, SOLCLIENT_SESSION_PROP_AUTHENTICATION_SCHEME, \
    SOLCLIENT_SESSION_PROP_USERNAME, SOLCLIENT_SESSION_PROP_PASSWORD, \
    SOLCLIENT_SESSION_PROP_SSL_CLIENT_CERTIFICATE_FILE, SOLCLIENT_SESSION_PROP_SSL_CLIENT_PRIVATE_KEY_FILE, \
    SOLCLIENT_SESSION_PROP_SSL_CLIENT_PRIVATE_KEY_FILE_PASSWORD, SOLCLIENT_SESSION_PROP_OAUTH2_ACCESS_TOKEN, \
    SOLCLIENT_SESSION_PROP_OIDC_ID_TOKEN, SOLCLIENT_SESSION_PROP_OAUTH2_ISSUER_IDENTIFIER, SOLCLIENT_FLOW_PROP_NO_LOCAL, \
    SOLCLIENT_SESSION_PROP_CLIENT_NAME, SOLCLIENT_SESSION_PROP_APPLICATION_DESCRIPTION, \
    SOLCLIENT_SESSION_PROP_KRB_SERVICE_NAME, SOLCLIENT_PROP_DISABLE_VAL, SOLCLIENT_FLOW_PROP_START_STATE, \
    SOLCLIENT_TRANSACTEDSESSION_PROP_REQUESTREPLY_TIMEOUT_MS
from solace.messaging.config.solace_properties import service_properties, authentication_properties, \
    transport_layer_properties, client_properties, transport_layer_security_properties, receiver_properties, \
    _legacy_properties, transactional_service_properties
from solace.messaging.config.solace_properties.authentication_properties import SCHEME_BASIC_USER_NAME, \
    SCHEME_BASIC_PASSWORD, SCHEME_SSL_CLIENT_CERT_FILE, SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE, \
    SCHEME_KERBEROS_USER_NAME, SCHEME_CLIENT_CERT_USER_NAME, \
    SCHEME, SCHEME_OAUTH2_ACCESS_TOKEN, SCHEME_OAUTH2_OIDC_ID_TOKEN
from solace.messaging.config.solace_constants.authentication_constants import AUTHENTICATION_SCHEME_KERBEROS, \
    AUTHENTICATION_SCHEME_CLIENT_CERT, AUTHENTICATION_SCHEME_BASIC, AUTHENTICATION_SCHEME_OAUTH2
from solace.messaging.config.solace_properties.service_properties import VPN_NAME
from solace.messaging.config.solace_properties.transport_layer_properties import HOST

flow_props = {SOLCLIENT_FLOW_PROP_BIND_ENTITY_ID: SOLCLIENT_FLOW_PROP_BIND_ENTITY_QUEUE,
              SOLCLIENT_FLOW_PROP_BIND_ENTITY_DURABLE: SOLCLIENT_PROP_ENABLE_VAL,
              SOLCLIENT_FLOW_PROP_ACKMODE: SOLCLIENT_FLOW_PROP_ACKMODE_CLIENT,
              SOLCLIENT_FLOW_PROP_ACTIVE_FLOW_IND: SOLCLIENT_PROP_ENABLE_VAL,
              SOLCLIENT_FLOW_PROP_START_STATE: SOLCLIENT_PROP_DISABLE_VAL}
# enable active flow indication default is disabled

end_point_props = {SOLCLIENT_ENDPOINT_PROP_ID: SOLCLIENT_ENDPOINT_PROP_QUEUE,
                   SOLCLIENT_ENDPOINT_PROP_DURABLE: SOLCLIENT_PROP_ENABLE_VAL,
                   SOLCLIENT_ENDPOINT_PROP_ACCESSTYPE: SOLCLIENT_ENDPOINT_PROP_ACCESSTYPE_EXCLUSIVE}

replay_prop = {SOLCLIENT_FLOW_PROP_REPLAY_START_LOCATION: SOLCLIENT_FLOW_PROP_REPLAY_START_LOCATION_BEGINNING}

mandatory_props = [HOST, VPN_NAME,
                   [SCHEME_BASIC_USER_NAME,
                    _legacy_properties._SCHEME_BASIC_USER_NAME_v1_2_1],
                   [SCHEME_BASIC_PASSWORD,
                    _legacy_properties._SCHEME_BASIC_PASSWORD_v1_2_1]]
kerberos_mandatory_props = [HOST, VPN_NAME]
certificate_mandatory_props = [HOST, VPN_NAME,
                               [SCHEME_SSL_CLIENT_CERT_FILE,
                                _legacy_properties._SCHEME_SSL_CLIENT_CERT_FILE_v1_2_1],
                               [SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE,
                                _legacy_properties._SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE_v1_2_1]]
oauth2_mandatory_props = [HOST, VPN_NAME, [SCHEME_OAUTH2_ACCESS_TOKEN, SCHEME_OAUTH2_OIDC_ID_TOKEN]]

DEFAULT_PROP = "property.is.default"
REQUIRED_PROPS = "property.is.required"
EXCLUDED_PROPS = "property.is.excluded"

scheme_basic_prop_map = {
    REQUIRED_PROPS: mandatory_props,
    EXCLUDED_PROPS: [
        SCHEME_CLIENT_CERT_USER_NAME,
        SCHEME_KERBEROS_USER_NAME
    ]
}

scheme_kerberos_prop_map = {
    REQUIRED_PROPS: kerberos_mandatory_props,
    EXCLUDED_PROPS: [
        SCHEME_BASIC_USER_NAME,
        SCHEME_CLIENT_CERT_USER_NAME
    ]
}

scheme_client_cert_prop_map = {
    REQUIRED_PROPS: certificate_mandatory_props,
    EXCLUDED_PROPS: [
        SCHEME_BASIC_USER_NAME,
        SCHEME_KERBEROS_USER_NAME
    ]
}

scheme_oauth2_prop_map = {
    REQUIRED_PROPS: oauth2_mandatory_props,
    EXCLUDED_PROPS: [
        SCHEME_BASIC_USER_NAME,
        SCHEME_CLIENT_CERT_USER_NAME,
        SCHEME_KERBEROS_USER_NAME
    ]
}

rectifiable_service_props = {
    SCHEME: {
        AUTHENTICATION_SCHEME_KERBEROS: scheme_kerberos_prop_map,
        AUTHENTICATION_SCHEME_BASIC: scheme_basic_prop_map,
        AUTHENTICATION_SCHEME_CLIENT_CERT: scheme_client_cert_prop_map,
        AUTHENTICATION_SCHEME_OAUTH2: scheme_oauth2_prop_map,
        DEFAULT_PROP: scheme_basic_prop_map
    }
}
CCSMP_SESSION_PROP_MAPPING = {
    transport_layer_properties.HOST: SOLCLIENT_SESSION_PROP_HOST,
    transport_layer_properties.CONNECTION_ATTEMPTS_TIMEOUT: SOLCLIENT_SESSION_PROP_CONNECT_TIMEOUT_MS,
    transport_layer_properties.CONNECTION_RETRIES: SOLCLIENT_SESSION_PROP_CONNECT_RETRIES,
    transport_layer_properties.CONNECTION_RETRIES_PER_HOST: SOLCLIENT_SESSION_PROP_CONNECT_RETRIES_PER_HOST,
    transport_layer_properties.RECONNECTION_ATTEMPTS: SOLCLIENT_SESSION_PROP_RECONNECT_RETRIES,
    transport_layer_properties.RECONNECTION_ATTEMPTS_WAIT_INTERVAL: SOLCLIENT_SESSION_PROP_RECONNECT_RETRY_WAIT_MS,
    transport_layer_properties.KEEP_ALIVE_INTERVAL: SOLCLIENT_SESSION_PROP_KEEP_ALIVE_INT_MS,
    transport_layer_properties.KEEP_ALIVE_WITHOUT_RESPONSE_LIMIT: SOLCLIENT_SESSION_PROP_KEEP_ALIVE_LIMIT,
    transport_layer_properties.SOCKET_INPUT_BUFFER_SIZE: SOLCLIENT_SESSION_PROP_SOCKET_RCV_BUF_SIZE,
    transport_layer_properties.SOCKET_OUTPUT_BUFFER_SIZE: SOLCLIENT_SESSION_PROP_SOCKET_SEND_BUF_SIZE,
    transport_layer_properties.SOCKET_TCP_OPTION_NO_DELAY: SOLCLIENT_SESSION_PROP_TCP_NODELAY,
    transport_layer_properties.COMPRESSION_LEVEL: SOLCLIENT_SESSION_PROP_COMPRESSION_LEVEL,
    transport_layer_security_properties.CERT_VALIDATED: SOLCLIENT_SESSION_PROP_SSL_VALIDATE_CERTIFICATE,
    transport_layer_security_properties.CERT_REJECT_EXPIRED: SOLCLIENT_SESSION_PROP_SSL_VALIDATE_CERTIFICATE_DATE,
    transport_layer_security_properties.CERT_VALIDATE_SERVERNAME: SOLCLIENT_SESSION_PROP_SSL_VALIDATE_CERTIFICATE_HOST,
    transport_layer_security_properties.EXCLUDED_PROTOCOLS: SOLCLIENT_SESSION_PROP_SSL_EXCLUDED_PROTOCOLS,
    transport_layer_security_properties.PROTOCOL_DOWNGRADE_TO: SOLCLIENT_SESSION_PROP_SSL_CONNECTION_DOWNGRADE_TO,
    transport_layer_security_properties.CIPHER_SUITES: SOLCLIENT_SESSION_PROP_SSL_CIPHER_SUITES,
    transport_layer_security_properties.TRUST_STORE_PATH: SOLCLIENT_SESSION_PROP_SSL_TRUST_STORE_DIR,
    transport_layer_security_properties.TRUSTED_COMMON_NAME_LIST: SOLCLIENT_SESSION_PROP_SSL_TRUSTED_COMMON_NAME_LIST,
    service_properties.VPN_NAME: SOLCLIENT_SESSION_PROP_VPN_NAME,
    service_properties.GENERATE_SENDER_ID: SOLCLIENT_SESSION_PROP_GENERATE_SENDER_ID,
    service_properties.GENERATE_RECEIVE_TIMESTAMPS: SOLCLIENT_SESSION_PROP_GENERATE_RCV_TIMESTAMPS,
    _legacy_properties._GENERATE_RECEIVE_TIMESTAMPS_v1_0_0: SOLCLIENT_SESSION_PROP_GENERATE_RCV_TIMESTAMPS,
    service_properties.GENERATE_SEND_TIMESTAMPS: SOLCLIENT_SESSION_PROP_GENERATE_SEND_TIMESTAMPS,
    _legacy_properties._GENERATE_SEND_TIMESTAMPS_v1_0_0: SOLCLIENT_SESSION_PROP_GENERATE_SEND_TIMESTAMPS,
    authentication_properties.SCHEME: SOLCLIENT_SESSION_PROP_AUTHENTICATION_SCHEME,
    authentication_properties.SCHEME_BASIC_USER_NAME: SOLCLIENT_SESSION_PROP_USERNAME,
    _legacy_properties._SCHEME_BASIC_USER_NAME_v1_2_1: SOLCLIENT_SESSION_PROP_USERNAME,
    authentication_properties.SCHEME_BASIC_PASSWORD: SOLCLIENT_SESSION_PROP_PASSWORD,
    _legacy_properties._SCHEME_BASIC_PASSWORD_v1_2_1: SOLCLIENT_SESSION_PROP_PASSWORD,
    authentication_properties.SCHEME_CLIENT_CERT_USER_NAME: SOLCLIENT_SESSION_PROP_USERNAME,
    authentication_properties.SCHEME_SSL_CLIENT_CERT_FILE: SOLCLIENT_SESSION_PROP_SSL_CLIENT_CERTIFICATE_FILE,
    _legacy_properties._SCHEME_SSL_CLIENT_CERT_FILE_v1_2_1: SOLCLIENT_SESSION_PROP_SSL_CLIENT_CERTIFICATE_FILE,
    authentication_properties.SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE: SOLCLIENT_SESSION_PROP_SSL_CLIENT_PRIVATE_KEY_FILE,
    _legacy_properties._SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE_v1_2_1: SOLCLIENT_SESSION_PROP_SSL_CLIENT_PRIVATE_KEY_FILE,
    authentication_properties.SCHEME_CLIENT_PRIVATE_KEY_FILE_PASSWORD: SOLCLIENT_SESSION_PROP_SSL_CLIENT_PRIVATE_KEY_FILE_PASSWORD,
    _legacy_properties._SCHEME_CLIENT_PRIVATE_KEY_FILE_PASSWORD_v1_2_1: SOLCLIENT_SESSION_PROP_SSL_CLIENT_PRIVATE_KEY_FILE_PASSWORD,
    authentication_properties.SCHEME_KERBEROS_USER_NAME: SOLCLIENT_SESSION_PROP_USERNAME,
    authentication_properties.SCHEME_OAUTH2_ACCESS_TOKEN: SOLCLIENT_SESSION_PROP_OAUTH2_ACCESS_TOKEN,
    authentication_properties.SCHEME_OAUTH2_OIDC_ID_TOKEN: SOLCLIENT_SESSION_PROP_OIDC_ID_TOKEN,
    authentication_properties.SCHEME_OAUTH2_ISSUER_IDENTIFIER: SOLCLIENT_SESSION_PROP_OAUTH2_ISSUER_IDENTIFIER,
    receiver_properties.PERSISTENT_NO_LOCAL_PUBLISHED_MESSAGES: SOLCLIENT_FLOW_PROP_NO_LOCAL,
    _legacy_properties._PERSISTENT_NO_LOCAL_PUBLISHED_MESSAGES_v1_0_0: SOLCLIENT_FLOW_PROP_NO_LOCAL,
    client_properties.NAME: SOLCLIENT_SESSION_PROP_CLIENT_NAME,
    client_properties.APPLICATION_DESCRIPTION: SOLCLIENT_SESSION_PROP_APPLICATION_DESCRIPTION,
    authentication_properties.KRB_SERVICE_NAME: SOLCLIENT_SESSION_PROP_KRB_SERVICE_NAME,
    transactional_service_properties.TRANSACTIONAL_SERVICE_REQUEST_TIMEOUT: SOLCLIENT_TRANSACTEDSESSION_PROP_REQUESTREPLY_TIMEOUT_MS
}

# The following dictionary is used to map the legacy properties to the current properties.
# This map is used only for those properties present in the CCSMP_SESSION_PROP_MAPPING.
# This mapping is useful for any methods that want to discern between the legacy and current property
# if both are provided to a configuration.
LEGACY_TO_CURRENT_CCSMP_SESSION_PROP_MAPPING = {
    _legacy_properties._GENERATE_RECEIVE_TIMESTAMPS_v1_0_0: service_properties.GENERATE_RECEIVE_TIMESTAMPS,
    _legacy_properties._GENERATE_SEND_TIMESTAMPS_v1_0_0: service_properties.GENERATE_SEND_TIMESTAMPS,
    _legacy_properties._PERSISTENT_NO_LOCAL_PUBLISHED_MESSAGES_v1_0_0: receiver_properties.PERSISTENT_NO_LOCAL_PUBLISHED_MESSAGES,
    _legacy_properties._SCHEME_BASIC_USER_NAME_v1_2_1: authentication_properties.SCHEME_BASIC_USER_NAME,
    _legacy_properties._SCHEME_BASIC_PASSWORD_v1_2_1: authentication_properties.SCHEME_BASIC_PASSWORD,
    _legacy_properties._SCHEME_SSL_CLIENT_CERT_FILE_v1_2_1: authentication_properties.SCHEME_SSL_CLIENT_CERT_FILE,
    _legacy_properties._SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE_v1_2_1: authentication_properties.SCHEME_SSL_CLIENT_PRIVATE_KEY_FILE,
    _legacy_properties._SCHEME_CLIENT_PRIVATE_KEY_FILE_PASSWORD_v1_2_1: authentication_properties.SCHEME_CLIENT_PRIVATE_KEY_FILE_PASSWORD
}
